/*
 * Copyright 2008, Krasimir S. Stefanov
 *
 * This file is part of laptop-mode-setup
 *
 * This program file is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program in a file named COPYING; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA
 *
 * Authors:
 * 	Krasimir S. Stefanov <lokiisyourmaster@gmail.com>
 */

#include <stdlib.h>
#include <gtk/gtk.h>
#include <glade/glade.h>
#include <string.h>

#include <glib.h>
#include <glib/gstdio.h>
#include <unistd.h>

#include <libintl.h>
#define _(String) dgettext (PACKAGE, String)
#define gettext_noop(String) String
#define N_(String) gettext_noop (String)

#define PACKAGE_VERSION "0.1"
#define PACKAGE_NAME "laptop-mode-setup"
#define PACKAGE "laptop-mode-setup"

#define T_BOOL 0
#define T_INT 1
#define T_STRING 2

#define PATH_LAPTOP_MODE_CONF "\"/etc/laptop-mode/laptop-mode.conf\""
#define PATH_ACPI_SUPPORT "\"/etc/default/acpi-support\""

static GtkWidget *window1 = NULL;
static GladeXML  *xml;

static void on_button1_clicked (GtkWidget *widget, gpointer user_data);
static void on_button2_clicked (GtkWidget *widget, gpointer user_data);
static void on_button3_clicked (GtkWidget *widget, gpointer user_data);
static void on_button4_clicked (GtkWidget *widget, gpointer user_data);
static void on_button5_clicked (GtkWidget *widget, gpointer user_data);
static void msgbox(gchar *message);
static gchar *shell_cmd(gchar *cmd);
static void set_option(gchar *name, gchar *widget, int type, gchar *file);
static void get_option(gchar *name, gchar *widget, int type, gchar *file);
static void read_options(void);

static void
on_button1_clicked (GtkWidget *widget, gpointer user_data) {
    // check if we are root
    if (g_strcmp0(g_strjoinv("", g_strsplit(shell_cmd("whoami"), "\n", -1)),g_strdup("root"))!=0) {
        msgbox(_("You must have administrative privileges to change the settings! (Run \"sudo laptop-mode-setup\")"));
        return;
    }

    shell_cmd("sudo mkdir -p /root/.laptop-mode-setup/");
    // backup the files if there are no backups
    if (g_access("/root/.laptop-mode-setup/laptop-mode.conf.default",F_OK)!=0){
        shell_cmd("sudo cp /etc/laptop-mode/laptop-mode.conf /root/.laptop-mode-setup/laptop-mode.conf.default");
    }

    if (g_access("/root/.laptop-mode-setup/acpi-support.default",F_OK)!=0){
        shell_cmd("sudo cp /etc/default/acpi-support /root/.laptop-mode-setup/acpi-support.default");
    }

    // set laptop-mode.conf options
    set_option("VERBOSE_OUTPUT","checkbutton1",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("ENABLE_LAPTOP_MODE_ON_BATTERY","checkbutton2",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("ENABLE_LAPTOP_MODE_ON_AC","checkbutton3",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("ENABLE_LAPTOP_MODE_WHEN_LID_CLOSED","checkbutton3",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("MINIMUM_BATTERY_CHARGE_PERCENT","spinbutton1",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("DISABLE_LAPTOP_MODE_ON_CRITICAL_BATTERY_LEVEL","checkbutton6",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("HD","entry1",T_STRING,PATH_LAPTOP_MODE_CONF);
    set_option("PARTITIONS","entry3",T_STRING,PATH_LAPTOP_MODE_CONF);
    set_option("ASSUME_SCSI_IS_SATA","checkbutton7",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("LM_BATT_MAX_LOST_WORK_SECONDS","spinbutton3",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("LM_AC_MAX_LOST_WORK_SECONDS","spinbutton5",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("CONTROL_READAHEAD","checkbutton8",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("LM_READAHEAD","spinbutton6",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_READAHEAD","spinbutton7",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("CONTROL_NOATIME","checkbutton9",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("USE_RELATIME","checkbutton10",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("CONTROL_HD_IDLE_TIMEOUT","checkbutton11",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("LM_AC_HD_IDLE_TIMEOUT_SECONDS","spinbutton2",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("LM_BATT_HD_IDLE_TIMEOUT_SECONDS","spinbutton4",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_HD_IDLE_TIMEOUT_SECONDS","spinbutton8",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("CONTROL_HD_POWERMGMT","checkbutton12",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("BATT_HD_POWERMGMT","spinbutton9",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("LM_AC_HD_POWERMGMT","spinbutton10",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_AC_HD_POWERMGMT","spinbutton11",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("CONTROL_HD_WRITECACHE","checkbutton13",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_AC_HD_WRITECACHE","checkbutton14",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_BATT_HD_WRITECACHE","checkbutton15",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("LM_HD_WRITECACHE","checkbutton16",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("CONTROL_MOUNT_OPTIONS","checkbutton17",T_BOOL,PATH_LAPTOP_MODE_CONF);
    set_option("LM_DIRTY_RATIO","spinbutton12",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_DIRTY_RATIO","spinbutton13",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("LM_DIRTY_BACKGROUND_RATIO","spinbutton14",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("NOLM_DIRTY_BACKGROUND_RATIO","spinbutton15",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("DEF_UPDATE","spinbutton16",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("DEF_XFS_AGE_BUFFER","spinbutton17",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("DEF_XFS_SYNC_INTERVAL","spinbutton18",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("DEF_XFS_BUFD_INTERVAL","spinbutton19",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("DEF_MAX_AGE","spinbutton20",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("XFS_HZ","spinbutton21",T_INT,PATH_LAPTOP_MODE_CONF);
    set_option("LM_SECONDS_BEFORE_SYNC","spinbutton22",T_INT,PATH_LAPTOP_MODE_CONF);

    // set acpi-support options
    set_option("SUSPEND_METHODS","entry2",T_STRING,PATH_ACPI_SUPPORT);
    set_option("ACPI_SLEEP","checkbutton5",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("ACPI_HIBERNATE","checkbutton18",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("ACPI_SLEEP_MODE","entry4",T_STRING,PATH_ACPI_SUPPORT);
    set_option("MODULES","entry5",T_STRING,PATH_ACPI_SUPPORT);
    set_option("MODULES_WHITELIST","entry6",T_STRING,PATH_ACPI_SUPPORT);
    set_option("SAVE_VBE_STATE","checkbutton19",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("VBESTATE","entry7",T_STRING,PATH_ACPI_SUPPORT);
    set_option("POST_VIDEO","checkbutton20",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("USE_DPMS","checkbutton21",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("HIBERNATE_MODE","entry8",T_STRING,PATH_ACPI_SUPPORT);
    set_option("LOCK_SCREEN","checkbutton22",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("STOP_SERVICES","entry9",T_STRING,PATH_ACPI_SUPPORT);
    set_option("RESTART_IRDA","checkbutton23",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("ENABLE_LAPTOP_MODE","checkbutton24",T_BOOL,PATH_ACPI_SUPPORT);
    set_option("SKIP_INTERFACES","entry10",T_STRING,PATH_ACPI_SUPPORT);
    set_option("XFS_HZ","spinbutton21",T_INT,PATH_ACPI_SUPPORT);


    if (system("sudo /etc/init.d/laptop-mode restart")!=0) {
        msgbox(_("Error while restarting laptop-mode!"));
    } else {
        msgbox(_("Done!"));
    }

    // re-read all options
    read_options();
}

static void read_options(void) {
    get_option("VERBOSE_OUTPUT","checkbutton1",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("ENABLE_LAPTOP_MODE_ON_BATTERY","checkbutton2",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("ENABLE_LAPTOP_MODE_ON_AC","checkbutton3",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("ENABLE_LAPTOP_MODE_WHEN_LID_CLOSED","checkbutton3",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("MINIMUM_BATTERY_CHARGE_PERCENT","spinbutton1",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("DISABLE_LAPTOP_MODE_ON_CRITICAL_BATTERY_LEVEL","checkbutton6",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("HD","entry1",T_STRING,PATH_LAPTOP_MODE_CONF);
    get_option("PARTITIONS","entry3",T_STRING,PATH_LAPTOP_MODE_CONF);
    get_option("ASSUME_SCSI_IS_SATA","checkbutton7",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("LM_BATT_MAX_LOST_WORK_SECONDS","spinbutton3",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("LM_AC_MAX_LOST_WORK_SECONDS","spinbutton5",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("CONTROL_READAHEAD","checkbutton8",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("LM_READAHEAD","spinbutton6",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_READAHEAD","spinbutton7",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("CONTROL_NOATIME","checkbutton9",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("USE_RELATIME","checkbutton10",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("CONTROL_HD_IDLE_TIMEOUT","checkbutton11",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("LM_AC_HD_IDLE_TIMEOUT_SECONDS","spinbutton2",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("LM_BATT_HD_IDLE_TIMEOUT_SECONDS","spinbutton4",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_HD_IDLE_TIMEOUT_SECONDS","spinbutton8",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("CONTROL_HD_POWERMGMT","checkbutton12",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("BATT_HD_POWERMGMT","spinbutton9",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("LM_AC_HD_POWERMGMT","spinbutton10",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_AC_HD_POWERMGMT","spinbutton11",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("CONTROL_HD_WRITECACHE","checkbutton13",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_AC_HD_WRITECACHE","checkbutton14",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_BATT_HD_WRITECACHE","checkbutton15",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("LM_HD_WRITECACHE","checkbutton16",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("CONTROL_MOUNT_OPTIONS","checkbutton17",T_BOOL,PATH_LAPTOP_MODE_CONF);
    get_option("LM_DIRTY_RATIO","spinbutton12",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_DIRTY_RATIO","spinbutton13",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("LM_DIRTY_BACKGROUND_RATIO","spinbutton14",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("NOLM_DIRTY_BACKGROUND_RATIO","spinbutton15",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("DEF_UPDATE","spinbutton16",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("DEF_XFS_AGE_BUFFER","spinbutton17",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("DEF_XFS_SYNC_INTERVAL","spinbutton18",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("DEF_XFS_BUFD_INTERVAL","spinbutton19",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("DEF_MAX_AGE","spinbutton20",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("XFS_HZ","spinbutton21",T_INT,PATH_LAPTOP_MODE_CONF);
    get_option("LM_SECONDS_BEFORE_SYNC","spinbutton22",T_INT,PATH_LAPTOP_MODE_CONF);

    get_option("SUSPEND_METHODS","entry2",T_STRING,PATH_ACPI_SUPPORT);
    get_option("ACPI_SLEEP","checkbutton5",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("ACPI_HIBERNATE","checkbutton18",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("ACPI_SLEEP_MODE","entry4",T_STRING,PATH_ACPI_SUPPORT);
    get_option("MODULES","entry5",T_STRING,PATH_ACPI_SUPPORT);
    get_option("MODULES_WHITELIST","entry6",T_STRING,PATH_ACPI_SUPPORT);
    get_option("SAVE_VBE_STATE","checkbutton19",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("VBESTATE","entry7",T_STRING,PATH_ACPI_SUPPORT);
    get_option("POST_VIDEO","checkbutton20",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("USE_DPMS","checkbutton21",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("HIBERNATE_MODE","entry8",T_STRING,PATH_ACPI_SUPPORT);
    get_option("LOCK_SCREEN","checkbutton22",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("STOP_SERVICES","entry9",T_STRING,PATH_ACPI_SUPPORT);
    get_option("RESTART_IRDA","checkbutton23",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("ENABLE_LAPTOP_MODE","checkbutton24",T_BOOL,PATH_ACPI_SUPPORT);
    get_option("SKIP_INTERFACES","entry10",T_STRING,PATH_ACPI_SUPPORT);
    get_option("XFS_HZ","spinbutton21",T_INT,PATH_ACPI_SUPPORT);

}

static void set_option(gchar *name, gchar *widget, int type, gchar *file) {
    switch (type) {
    case T_BOOL:
        if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget( xml,  widget)))) {
            shell_cmd(g_strconcat("sudo sed -i s/",name,"=.*/",name,"=1/g ",file,"",NULL));
        } else {
            shell_cmd(g_strconcat("sudo sed -i s/",name,"=.*/",name,"=0/g ",file,"",NULL));
        }
        break;
    case T_INT:
        shell_cmd(g_strconcat("sudo sed -i s/",name,"=.*/",name,"=",
                              g_strdup_printf("%d",gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(glade_xml_get_widget( xml,  widget)))),
                              "/g ",file,"",NULL));
        break;
    case T_STRING:
        shell_cmd(g_strconcat("sudo sed -i s/",name,"=.*/",name,"=\"\\\"",
                              g_strjoinv("\\\\/", g_strsplit(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,widget))), "/", -1)),
                              "\\\"\"/g ",file,"",NULL));
        break;
    }
}
static void get_option(gchar *name, gchar *widget, int type, gchar *file) {
    gchar *value = NULL;
    value = shell_cmd(g_strconcat("cat ",file," | grep ^",name,"= | cut -d= -f2",NULL));
    switch (type) {
    case T_BOOL:
        if (g_strcmp0(g_strjoinv("", g_strsplit(value, "\n", -1)),"true")==0) {
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, widget)),TRUE);
        } else if(g_strcmp0(g_strjoinv("", g_strsplit(value, "\n", -1)),"false")==0) {
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, widget)),FALSE);
        } else if (atoi(value)==1) {
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, widget)),TRUE);
        } else {
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml, widget)),FALSE);
        }


        break;
    case T_INT:
        gtk_spin_button_set_value(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,widget)),atoi(value));
        break;
    case T_STRING:
        gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,widget)),
                           //remove new lines and double quotes
                           g_strjoinv("", g_strsplit(g_strjoinv("", g_strsplit(value, "\"", -1)), "\n", -1))
                          );
        break;
    }
    g_free(value);
}

static void
on_button2_clicked (GtkWidget *widget, gpointer user_data) {
    if (g_strcmp0(g_strjoinv("", g_strsplit(shell_cmd("whoami"), "\n", -1)),g_strdup("root"))!=0) {
        msgbox(_("You must have administrative privileges to change the settings! (Run \"sudo laptop-mode-setup\")"));
        return;
    }

    if (g_access("/root/.laptop-mode-setup/laptop-mode.conf.default",F_OK)==0){
        shell_cmd("sudo cp /root/.laptop-mode-setup/laptop-mode.conf.default /etc/laptop-mode/laptop-mode.conf");
        read_options();
        msgbox(_("The file /etc/laptop-mode/laptop-mode.conf has been restored successfully!"));
    } else {
        msgbox(_("There is no backup! \n(This is perfectly normal if you run this program for the first time)"));
    }
}

static void
on_button3_clicked (GtkWidget *widget, gpointer user_data) {
    gtk_main_quit();
}

static void
on_button4_clicked (GtkWidget *widget, gpointer user_data) {
    if (g_strcmp0(g_strjoinv("", g_strsplit(shell_cmd("whoami"), "\n", -1)),g_strdup("root"))!=0) {
        msgbox(_("You must have administrative privileges to change the settings! (Run \"sudo laptop-mode-setup\")"));
        return;
    }

    if (g_access("/root/.laptop-mode-setup/acpi-support.default",F_OK)==0){
        shell_cmd("sudo cp /root/.laptop-mode-setup/acpi-support.default /etc/default/acpi-support");
        read_options();
        msgbox(_("The file /etc/default/acpi-support has been restored successfully!"));
    } else {
        msgbox(_("There is no backup! \n(This is perfectly normal if you run this program for the first time)"));
    }
}

static void
on_button5_clicked (GtkWidget *widget, gpointer user_data) {
    read_options();
    msgbox(_("All options reloaded!"));
}

static void msgbox(gchar *message) {
    GtkWidget *dialog = NULL;
    dialog = gtk_message_dialog_new (GTK_WINDOW (window1), GTK_DIALOG_MODAL, GTK_MESSAGE_INFO, GTK_BUTTONS_CLOSE, "%s",message);
    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
    gtk_dialog_run (GTK_DIALOG (dialog));
    gtk_widget_destroy (dialog);
}

static gchar *shell_cmd(gchar *cmd) {
    gchar *stdout = NULL, *command = NULL;
//    gchar **split = NULL;
    if (cmd != NULL) {
        command = g_strconcat("/bin/sh -c \"",g_strescape(cmd,NULL),"\"", NULL);
        g_spawn_command_line_sync(command,&stdout,NULL,NULL,NULL);
//        printf("\n%s\n-----------------------------------------------------------",command);
//        split = g_strsplit(stdout,"\n",2);
    } else return NULL;
    g_free(command);
    return g_strdup(stdout);
}

int main (int argc, char *argv[]) {
    gtk_init (&argc, &argv);
    /* load the interface */
    xml = glade_xml_new ("laptop-mode-setup.glade", NULL, PACKAGE_NAME);

    /* Have the delete event (window close) end the program */
    window1 = glade_xml_get_widget (xml, "window1");

    /* connect the signals in the interface */
    g_signal_connect (G_OBJECT (window1), "delete_event", G_CALLBACK (gtk_main_quit), NULL);
    g_signal_connect (G_OBJECT (glade_xml_get_widget (xml, "button1")), "clicked", G_CALLBACK (on_button1_clicked), NULL);
    g_signal_connect (G_OBJECT (glade_xml_get_widget (xml, "button2")), "clicked", G_CALLBACK (on_button2_clicked), NULL);
    g_signal_connect (G_OBJECT (glade_xml_get_widget (xml, "button3")), "clicked", G_CALLBACK (on_button3_clicked), NULL);
    g_signal_connect (G_OBJECT (glade_xml_get_widget (xml, "button4")), "clicked", G_CALLBACK (on_button4_clicked), NULL);
    g_signal_connect (G_OBJECT (glade_xml_get_widget (xml, "button5")), "clicked", G_CALLBACK (on_button5_clicked), NULL);

//    glade_xml_signal_autoconnect(xml);
    gtk_widget_show_all(window1);
    read_options();

    /* start the event loop */
    gtk_main ();

    return 0;
}
